"""
@author: Sumit Kumar Singh
"""

#Import Required Packages
import numpy as np
import matplotlib.pyplot as plt

########### import IG beam module (IGbeam.py)##############
import IGbeam
#############################################

###### Uncomment the below to set input parameters from command line (p,m,beam) ########
# =============================================================================
# print("enter mode number to generate IG beam")
# p=int(input("p="))
# m=int(input("m="))
# beam=int(input('enter 2 for even Ince-Gaussian beam or Enter1 for odd Gauss ince beam== '))
# =============================================================================
#### Parameters hardcoded into the code itself (p,m,beam) (Fig.1d)
p=8
m=6
beam=2                      #choose 1 for odd beam
#######Change values as per requirement#######

L=0.01                                       #size of meshgrid
N=1200                                       #dimension of meshgrid
w0=0.003                                     #Beam waist                           
z=0                                          #Propagation Distance
b=0.9999955                                  #for IG beam
#b=0                                         #(f0=1)for Hermite Gaussian Beam
#b=0.999999999                               #(f0 =0) for Laguerre gaussian beam
f = 100                                      #frequency of carrier wave

def Intensity():
    A1=IGbeam.incegauss(p,m,beam,b,L,N,w0,z)
    I1=abs(A1)**2               #INTENSITY of Ince-gaussian beam
    #Normalization
    I=(I1-np.min(I1))/(np.max(I1)-np.min(I1))
    return I;
    
def Phase():
    P1=IGbeam.incegauss(p,m,beam,b,L,N,w0,z)
    phs=np.angle(P1)      #Phase of Ince-Gaussian Beam
    return phs;

def Hologram(f):                         #select f as carrier frequency  e.g.500
    #adding carrier frequency
    H1=IGbeam.incegauss(p,m,beam,b,L,N,w0,z)*car(f)

    phs1=np.angle(H1)                   #carrier frequency added phase
    hol=(((phs1-np.min(phs1))/(np.max(phs1)-np.min(phs1))) * 255).astype('uint8')
    return hol;

def car(f):
    x,y=np.meshgrid(np.linspace(-L,L,N),np.linspace(-L,L,N))
    carr=np.exp(1j*x*f/L)           #select carrier frequency(f) (e.g.500)
    return carr


def Helical(f):
   hel=IGbeam.incegauss(p,m,2,b,L,N,w0,z)+ IGbeam.incegauss(p,m,1,b,L,N,w0,z)
   #Intensity calculation
   I1=abs(hel)**2                       #INTENSITY of Ince-gaussian beam
    #Normalization of Intensity
   I=(I1-np.min(I1))/(np.max(I1)-np.min(I1))
   
   #phase calculation
   phs=np.angle(hel)
   H1=hel*car(f)
   phs1=np.angle(H1)                   #carrier frequency added phase
   hol=(((phs1-np.min(phs1))/(np.max(phs1)-np.min(phs1))) * 255).astype('uint8')
   
   return I,phs,hol

# To generate superposed states (fig.12a) Change p,m,beam,b for other states
def superposition(f):
    #Calculate Intensity
    A1=IGbeam.incegauss(3,3,2,b,L,N,w0,z) + IGbeam.incegauss(3,1,2,b,L,N,w0,z)
    I1=abs(A1)**2               #INTENSITY of Ince-gaussian beam
    I=(I1-np.min(I1))/(np.max(I1)-np.min(I1))
    # Calculate Hologram
    H1=(IGbeam.incegauss(3,3,2,b,L,N,w0,z) + IGbeam.incegauss(3,1,2,b,L,N,w0,z))*car(f)
    phs1=np.angle(H1)                   #carrier frequency added phase
    hol=(((phs1-np.min(phs1))/(np.max(phs1)-np.min(phs1))) * 255).astype('uint8')
    return I, hol;


#Orthogonality constant calculation
def ortho():
    #Checking the orthogonality of ince gauss beam   
    ortho=IGbeam.incegauss(p,m,beam,b,L,N,w0,z)*np.conj(IGbeam.incegauss(p,m,beam,b,L,N,w0,z))
    orthconstant=((abs(np.sum(ortho)*((2*L)/(N-1))**2)))       #print orthogonality relation between two specific beams defined above in ortho
    return orthconstant;
    
#plot Intensity, phase and hologram (Fig.9)

plt.imshow(Intensity(),cmap='gray')
plt.colorbar()
plt.show()

plt.imshow(Phase(),cmap='gray')
plt.colorbar()
plt.show()


plt.imshow(Hologram(f),cmap='gray')
plt.axis("off")
plt.show()


# plot Helical Ince-Gaussian beam  (Fig.10)
# =============================================================================
# I,phs,hol=Helical(100)
# plt.imshow(I,cmap='gray')
# plt.colorbar()
# plt.show()
# 
# plt.imshow(phs,cmap='gray')
# plt.colorbar()
# plt.show()
# 
# plt.imshow(hol,cmap='gray')
# plt.axis("off")
# plt.show()
# =============================================================================

# To generate superposition states of Ince-Gaussian beam (Fig.12)
# =========================================================================
# intensity, hol = superposition(f)
# plt.imshow(intensity,cmap='gray')
# plt.show()
# plt.imshow(hol,cmap='gray')
# plt.axis("off")
# plt.show()
#=========================================================================


print("Congratulations!! Intensity,phase and Hologram of required IG beam have been Successfully generated.")
